/*
 * Sweep, a sound wave editor.
 *
 * Copyright (C) 2000-2002 Conrad Parker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#include <glib.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "callbacks.h"

#include <sweep/sweep_i18n.h>

#include <sweep/sweep_types.h>
#include <sweep/sweep_undo.h>
#include <sweep/sweep_sounddata.h>
#include <sweep/sweep_sample.h>
#include <sweep/sweep_typeconvert.h>

#include "sample.h"
#include "interface.h"
#include "edit.h"
#include "sample-display.h"
#include "play.h"
#include "driver.h"
#include "undo_dialog.h"
#include "paste_dialogs.h"

#include "../pixmaps/loop.xpm"
#include "../pixmaps/record.xpm"
#include "../pixmaps/record_stop.xpm"

#define DEBUG

extern GtkStyle * style_wb;
extern GtkStyle * style_LCD;
extern GtkStyle * style_light_grey;
extern GtkStyle * style_dark_grey;


static GtkWidget * rec_dialog = NULL;

static sw_handle * rec_handle = NULL;

static gboolean rec_prepared = FALSE;

static gboolean rec_looping = FALSE;

static void
stop_recording (sw_sample * sample)
{
  if (rec_handle != NULL) {
    device_close (rec_handle);
  }

  sample->recording = FALSE;

  rec_prepared = FALSE;
}

static void
prepare_recording (sw_sample * sample)
{
  rec_handle = device_open (O_RDONLY);

  if (rec_handle == NULL) return;

  device_setup (rec_handle, sample);

  rec_prepared = TRUE;
}

static struct timeval tv_instant = {0, 0};

static void
do_record_regions (sw_sample * sample)
{
  sw_sounddata * sounddata = sample->sounddata;
  sw_format * f = sounddata->format;
  fd_set fds;
  GList * gl;
  sw_sel * sel;
  sw_framecount_t sel_total, run_total;
  sw_framecount_t offset, remaining, n, count;
  gpointer d;
  gint percent;

  /*sw_audio_t rbuf[1024];*/

  gboolean active = TRUE;

  sample->recording = TRUE;

  if (!rec_prepared)
    prepare_recording (sample);

  if (rec_handle == NULL) goto done;

  sel_total = sounddata_selection_nr_frames (sounddata);

  do {
  run_total = 0;
  for (gl = sounddata->sels; active && gl; gl = gl->next) {
    sel = (sw_sel *)gl->data;
    
    offset = 0;
    remaining = sel->sel_end - sel->sel_start;
    
    while (active && remaining > 0) {
      g_mutex_lock (sample->ops_mutex);
      
      if (sample->edit_state == SWEEP_EDIT_STATE_CANCEL ||
	  sample->recording == FALSE) {
	active = FALSE;
      } else {
	FD_ZERO (&fds);
	FD_SET (rec_handle->driver_fd, &fds);
	
	if (select (rec_handle->driver_fd+1, &fds, NULL, NULL, &tv_instant) == 0);

	d = sounddata->data +
	  (int)frames_to_bytes (f, sel->sel_start + offset);

	n = MIN(remaining, 1024);

	count = n * f->channels;

	device_read (rec_handle, d, count);

	remaining -= n;
	offset += n;
	
	run_total += n;
	percent = run_total / (sel_total / 100);
	sample_set_progress_percent (sample, percent);

#ifdef DEBUG
	/*
	g_print ("completed %d / %d frames, %d%%\n", run_total, sel_total,
		 percent);
	*/
#endif

      }

      g_mutex_unlock (sample->ops_mutex);
    }
  }
  } while (rec_looping);

 done:
  stop_recording (sample);
}

static void
do_record_regions_thread (sw_op_instance * inst)
{
  sw_sample * sample = inst->sample;

  sw_edit_buffer * old_eb;
  paste_over_data * p;

  old_eb = edit_buffer_from_sample (sample);

  p = paste_over_data_new (old_eb, old_eb);
  inst->redo_data = inst->undo_data = p;
  set_active_op (sample, inst);

  do_record_regions (sample);

  if (sample->edit_state == SWEEP_EDIT_STATE_BUSY) {
    p->new_eb = edit_buffer_from_sample (sample);
    
    register_operation (sample, inst);
  }
}

static sw_operation record_regions_op = {
  SWEEP_EDIT_MODE_FILTER,
  (SweepCallback)do_record_regions_thread,
  (SweepFunction)NULL,
  (SweepCallback)undo_by_paste_over,
  (SweepFunction)paste_over_data_destroy,
  (SweepCallback)redo_by_paste_over,
  (SweepFunction)paste_over_data_destroy
};

static void
rec_loop_toggled_cb (GtkWidget * widget, gpointer data)
{
  sw_sample * sample = (sw_sample *)data;
  gboolean active;

  active = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget));
  /*  sample_set_looping (view->sample, active);*/
  rec_looping = active;
}

static void
prepare_cb (GtkWidget * widget, gpointer data)
{
  sw_sample * sample = (sw_sample *)data;

  if (!sample->recording) {
    prepare_recording (sample);
  }
}

void
record_cb (GtkWidget * widget, gpointer data)
{
  sw_sample * sample = (sw_sample *)data;

  if (sample->recording) {
    stop_recording (sample);
  } else {
    schedule_operation (sample, "Record", &record_regions_op, NULL);
  }
}

void
record_stop_cb (GtkWidget * widget, gpointer data)
{
  sw_sample * s = (sw_sample *)data;

  stop_recording (s);
}


static void
rec_dialog_destroy (void)
{
  rec_dialog = NULL;
}

static void
rec_dialog_cancel_cb (GtkWidget * widget, gpointer data)
{
  GtkWidget * dialog;

  dialog = gtk_widget_get_toplevel (widget);
  gtk_widget_hide (dialog);
}

typedef enum {
  SW_TOOLBAR_BUTTON,
  SW_TOOLBAR_TOGGLE_BUTTON,
  SW_TOOLBAR_RADIO_BUTTON,
} sw_toolbar_button_type;

static GtkWidget *
create_pixmap_button (GtkWidget * widget, gchar ** xpm_data,
		      const gchar * tip_text, GtkStyle * style,
		      sw_toolbar_button_type button_type,
		      GtkSignalFunc clicked,
		      GtkSignalFunc pressed, GtkSignalFunc released,
		      gpointer data)
{
  GtkWidget * pixmap;
  GtkWidget * button;
  GtkTooltips * tooltips;

  switch (button_type) {
  case SW_TOOLBAR_TOGGLE_BUTTON:
    button = gtk_toggle_button_new ();
    break;
  case SW_TOOLBAR_RADIO_BUTTON:
    button = gtk_radio_button_new (NULL);
    break;
  case SW_TOOLBAR_BUTTON:
  default:
    button = gtk_button_new ();
    break;
  }

  if (xpm_data != NULL) {
    pixmap = create_widget_from_xpm (widget, xpm_data);
    gtk_widget_show (pixmap);
    gtk_container_add (GTK_CONTAINER (button), pixmap);
  }

  if (tip_text != NULL) {
    tooltips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tooltips, button, tip_text, NULL);
  }

  if (style != NULL) {
    gtk_widget_set_style (button, style);
  }

  if (clicked != NULL) {
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC(clicked), data);
  }

  if (pressed != NULL) {
    gtk_signal_connect (GTK_OBJECT(button), "pressed",
			GTK_SIGNAL_FUNC(pressed), data);
  }

  if (released != NULL) {
    gtk_signal_connect (GTK_OBJECT(button), "released",
			GTK_SIGNAL_FUNC(released), data);
  }

  return button;
}

#if 0
static GtkWidget *
create_pixmap_button (GtkWidget * widget, gchar ** xpm_data,
		      const gchar * label_text, const gchar * tip_text,
		      GtkSignalFunc clicked, gpointer data)
{
  GtkWidget * hbox;
  GtkWidget * label;
  GtkWidget * pixmap;
  GtkWidget * button;
  GtkTooltips * tooltips;

  button = gtk_button_new ();

  hbox = gtk_hbox_new (FALSE, 2);
  gtk_container_add (GTK_CONTAINER(button), hbox);
  gtk_container_set_border_width (GTK_CONTAINER(button), 8);
  gtk_widget_show (hbox);

  if (xpm_data != NULL) {
    pixmap = create_widget_from_xpm (widget, xpm_data);
    gtk_box_pack_start (GTK_BOX(hbox), pixmap, FALSE, FALSE, 8);
    gtk_widget_show (pixmap);
  }

  if (label_text != NULL) {
    label = gtk_label_new (label_text);
    gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 8);
    gtk_widget_show (label);
  }

  if (tip_text != NULL) {
    tooltips = gtk_tooltips_new ();
    gtk_tooltips_set_tip (tooltips, button, tip_text, NULL);
  }

  if (clicked != NULL) {
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
			GTK_SIGNAL_FUNC(clicked), data);
  }

  return button;
}
#endif

void
rec_dialog_create (sw_sample * sample)
{
  GtkWidget * handlebox;
  GtkWidget * main_vbox;
  GtkWidget * hbox;
  GtkWidget * tool_hbox;
  GtkWidget * button;

  GtkAccelGroup * accel_group;

  if (rec_dialog == NULL) {
    rec_dialog = gtk_dialog_new ();
    gtk_window_set_wmclass(GTK_WINDOW(rec_dialog), "rec_dialog", "Sweep");
    gtk_window_set_title(GTK_WINDOW(rec_dialog), _("Sweep: Record"));
    gtk_window_set_policy(GTK_WINDOW(rec_dialog), FALSE, FALSE, FALSE);
    gtk_window_position(GTK_WINDOW(rec_dialog), GTK_WIN_POS_MOUSE);
    gtk_container_border_width (GTK_CONTAINER(rec_dialog), 8);

    accel_group = gtk_accel_group_new ();
    gtk_window_add_accel_group (GTK_WINDOW(rec_dialog), accel_group);

    gtk_signal_connect(GTK_OBJECT(rec_dialog), "destroy",
		       (GtkSignalFunc) rec_dialog_destroy, NULL);

    gtk_accel_group_add (accel_group, GDK_w, GDK_CONTROL_MASK, GDK_NONE,
			 GTK_OBJECT(rec_dialog), "hide");

    main_vbox = GTK_DIALOG(rec_dialog)->vbox;

    handlebox = gtk_handle_box_new ();
    gtk_box_pack_start (GTK_BOX (main_vbox), handlebox, FALSE, TRUE, 0);
    gtk_widget_show (handlebox);
    
    gtk_widget_set_style (handlebox, style_dark_grey);

    hbox = gtk_hbox_new (FALSE, 8);
    gtk_container_add (GTK_CONTAINER (handlebox), hbox);
    gtk_widget_show (hbox);
    
    gtk_widget_set_usize (hbox, -1, 24);

    tool_hbox = gtk_hbox_new (TRUE, 0);
    gtk_box_pack_start (GTK_BOX (hbox), tool_hbox, FALSE, TRUE, 0);
    gtk_widget_show (tool_hbox);

    button = create_pixmap_button (rec_dialog, loop_xpm,
				   _("Loop mode recording (toggle)"),
				   style_light_grey, SW_TOOLBAR_TOGGLE_BUTTON,
				   rec_loop_toggled_cb, NULL, NULL, sample);
    
    gtk_signal_handler_block_by_data (GTK_OBJECT(button), sample);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button),
				  rec_looping);
    gtk_signal_handler_unblock_by_data (GTK_OBJECT(button), sample);
    
    gtk_box_pack_start (GTK_BOX (tool_hbox), button, FALSE, TRUE, 0);
    gtk_widget_show (button);
    

    button = create_pixmap_button (rec_dialog, NULL, _("Prepare"),
				   style_light_grey, SW_TOOLBAR_TOGGLE_BUTTON,
				   prepare_cb, NULL, NULL, sample);
    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
    gtk_widget_show (button);

    gtk_signal_handler_block_by_data (GTK_OBJECT(button), sample);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button),
				  rec_prepared);
    gtk_signal_handler_unblock_by_data (GTK_OBJECT(button), sample);

    button = create_pixmap_button (rec_dialog, record_xpm, _("Record"),
				   style_light_grey, SW_TOOLBAR_TOGGLE_BUTTON,
				   record_cb, NULL, NULL, sample);
    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
    gtk_widget_show (button);
#if 0
    gtk_widget_add_accelerator (button, "clicked", accel_group,
				GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#endif

    gtk_signal_handler_block_by_data (GTK_OBJECT(button), sample);
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(button),
				  sample->recording);
    gtk_signal_handler_unblock_by_data (GTK_OBJECT(button), sample);


    button = create_pixmap_button (rec_dialog, record_stop_xpm, _("Stop"),
				   style_light_grey, SW_TOOLBAR_BUTTON,
				   record_stop_cb, NULL, NULL, sample);
    gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
    gtk_widget_show (button);
#if 0
    gtk_widget_add_accelerator (button, "clicked", accel_group,
				GDK_r, GDK_CONTROL_MASK, GTK_ACCEL_VISIBLE);
#endif

    /* Cancel */

    button = gtk_button_new_with_label (_("Close"));
    GTK_WIDGET_SET_FLAGS (GTK_WIDGET (button), GTK_CAN_DEFAULT);
    gtk_box_pack_start (GTK_BOX (GTK_DIALOG(rec_dialog)->action_area),
			button, FALSE, FALSE, 0);
    gtk_widget_show (button);
    gtk_signal_connect (GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC (rec_dialog_cancel_cb),
			NULL);

    gtk_widget_grab_default (button);

  }

  if (!GTK_WIDGET_VISIBLE(rec_dialog)) {
    gtk_widget_show (rec_dialog);
  } else {
    gdk_window_raise (rec_dialog->window);
  }

}
